<?php

namespace Drupal\views_filters_populate\Plugin\views\filter;

use Drupal\views\Plugin\views\filter\FilterPluginBase;
use Drupal\views\Plugin\views\filter\StringFilter;
use Drupal\views\Plugin\views\filter\NumericFilter;
use Drupal\Core\Form\FormStateInterface;

/**
 * Filter handler which allows to search on multiple fields.
 *
 * @ingroup views_field_handlers
 *
 * @ViewsFilter("views_filters_populate")
 */
class Populate extends FilterPluginBase {

  private function isFilterSupported(FilterPluginBase $filter) {
    if ($filter instanceof StringFilter) {
      return TRUE;
    }

    if ($filter instanceof NumericFilter) {
      return TRUE;
    }

    return FALSE;
  }

  protected function defineOptions() {
    $options = parent::defineOptions();
    $options['filters'] = ['default' => []];

    return $options;
  }

  public function buildOptionsForm(&$form, FormStateInterface $form_state) {
    parent::buildOptionsForm($form, $form_state);

    $options = [];
    foreach ($this->displayHandler->getHandlers('filter') as $name => $filter) {
      if ($filter != $this && $this->isFilterSupported($filter) && !$filter->options['exposed']) {
        $options[$name] = $filter->adminLabel(TRUE);
      }
    }

    $form['filters'] = [
      '#type' => 'select',
      '#title' => $this->t('Available filters'),
      '#description' => $this->t("Only supported <i><b>non exposed</b></i> filters that accept a string value are shown here."),
      '#multiple' => TRUE,
      '#options' => $options,
      '#default_value' => $this->options['filters'],
    ];
  }

  protected function valueValidate($form, FormStateInterface $form_state) {
    $filters = $form_state->getValue(['options', 'filters']);
    if (empty($form['filters']['#options'])) {
      $form_state->setErrorByName('options][filters', $this->t('You have to add <b>supported non exposed</b> filters to be able to use this filter.'));
    }
    elseif (empty($filters)) {
      $form_state->setErrorByName('options][filters', $this->t('You have to select at least one filter to be populated.'));
    }
  }

  public function preQuery() {
    // start with default value
    $value = $this->value;

    // If exposed, take data from input, as this hasn't reached the views
    // object yet
    if ($this->options['exposed']) {
      $input = $this->view->getExposedInput();
      $identifier = $this->options['expose']['identifier'];
      if (isset($input[$identifier])) {
        $value = $input[$identifier];
      }
    }

    foreach ($this->options['filters'] as $id) {
      $filter = $this->view->filter[$id];
      if (empty($value)) {
        unset($this->view->filter[$id]);
      }
      else {
        if ($filter instanceof StringFilter) {
          $filter->value = $value;
        }
        if ($filter instanceof NumericFilter) {
          $filter->value['value'] = $value;
        }
      }
    }
  }

  public function query() {
    // do nothing
  }

  /**
   * {@inheritdoc}
   */
  public function validate() {
    $errors = parent::validate();

    // Only perform these validatiosn if the query is not being built
    // @see ViewExecutable::build()
    if (empty($this->view->build_info)) {
      $filters = $this->displayHandler->getHandlers('filter');
      foreach ($this->options['filters'] as $id) {
        // Make sure are populate filters are properly configured/available
        if (!isset($filters[$id])) {
          $errors[] = $this->t('%display: Filter %filter set in %name is not available.', ['%filter' => $id, '%name' => $this->adminLabel(), '%display' => $this->displayHandler->display['display_title']]);
        }
        else {
          $filter = $filters[$id];

          if ($filter->options['exposed']) {
            $errors[] = $this->t('%display: Filter %filter set in %name is not usable when exposed.', ['%filter' => $filter->adminLabel(), '%name' => $this->adminLabel(), '%display' => $this->displayHandler->display['display_title']]);
          }
        }
      }
    }

    return $errors;
  }

  /**
   * Provide a simple textfield for equality
   *
   * @see \Drupal\views\Plugin\views\filter\StringFilter;
   */
  protected function valueForm(&$form, FormStateInterface $form_state) {

    if ($exposed = $form_state->get('exposed')) {
      $identifier = $this->options['expose']['identifier'];
    }

    $form['value'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Value'),
      '#size' => 30,
      '#default_value' => $this->value,
    ];

    if (!empty($this->options['expose']['placeholder'])) {
      $form['value']['#attributes']['placeholder'] = $this->options['expose']['placeholder'];
    }

    $user_input = $form_state->getUserInput();
    if ($exposed && !isset($user_input[$identifier])) {
      $user_input[$identifier] = $this->value;
      $form_state->setUserInput($user_input);
    }

    if (!isset($form['value'])) {
      // Ensure there is something in the 'value'.
      $form['value'] = [
        '#type' => 'value',
        '#value' => NULL,
      ];
    }
  }
}
